using System;
using System.Windows.Forms;
using System.Drawing;

namespace TicTacToe
{
    public enum TTTStatus { Empty, Cross, Circle, Undefined }

    public class TTTField : Control
    {
        private TTTStatus status;
        private bool locked;
   
        public TTTField() : this(TTTStatus.Empty)
        {
        }

        public TTTField(TTTStatus status)
        {
            this.status = status;
            locked = false;
            Size = new Size(50, 50);

            SetStyle(ControlStyles.UserPaint, true);
            SetStyle(ControlStyles.ResizeRedraw, true);
        }


        /***********************************
         * Eigenschaften
         */

        public TTTStatus Status
        {
            get { return status; }
            set { if (!locked) status = value; }
        }

        public bool Locked
        {
            get { return locked; }
            set { locked = value; }
        }


        /***********************************
         * Gren-Management
         */

        public override Size GetPreferredSize(Size proposedSize)
        {
            return new Size(50, 50);
        }

        public override Size MinimumSize
        {
            get
            {
                return new Size(50, 50);
            }
        }


        /***********************************
         * Zeichnen
         */
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            
            Graphics gc = e.Graphics;
            Rectangle rect = ClientRectangle;
                
            gc.FillRectangle(Brushes.AntiqueWhite, rect);
            gc.DrawRectangle(new Pen(Color.Black, 1), rect.X, rect.Y,
                                                      rect.Width-1,rect.Height-1);

            switch (status)
            {
                case TTTStatus.Cross:
                    gc.DrawLine(Pens.Black,
                      new Point(rect.X + 5, rect.Y + 5),
                      new Point(rect.X + rect.Width - 5, rect.Y + rect.Height - 5));
                    gc.DrawLine(Pens.Black,
                      new Point(rect.X + rect.Width - 5, rect.Y + 5),
                      new Point(rect.X + 5, rect.Y + rect.Height - 5));
                    break;
                case TTTStatus.Circle:
                    gc.DrawEllipse(Pens.Black, rect.X + 5, rect.Y + 5,
                                               rect.Width - 10, rect.Height - 10);
                    break;
            }
            
        }


        /***********************************
         * Ereignisbehandlung
         */
        protected override void OnClick(EventArgs e)
        {
            if (!locked)
            {
                status++;
                if (status == TTTStatus.Undefined)
                    status = TTTStatus.Empty;

                this.Refresh();
            }

            base.OnClick(e);
        }
    }
}
