using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Drawing.Printing;
using System.Text;
using System.Windows.Forms;

namespace Drucken
{
    public partial class Form1 : Form
    {
        Image img;
        OpenFileDialog openFileDialog1;
        SaveFileDialog saveFileDialog1;
        
        public Form1()
        {
            InitializeComponent();
            ResizeRedraw = true;
            img = null;
 
            openFileDialog1 = new OpenFileDialog();
            openFileDialog1.InitialDirectory = ".";
            openFileDialog1.Filter = "Windows Bitmap (*.bmp) | *.bmp"
                                   + "| JPEG (*.jpg) | *.jpg"
                                   + "| PNG (*.png) | *.png"
                                   + "| Grafikdatei (*.bmp;*.jpg;*.png;) | *.bmp;*.jpg;*.png;";
            openFileDialog1.FilterIndex = 4;
            openFileDialog1.Multiselect = false;
            openFileDialog1.RestoreDirectory = true;
            
            saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.InitialDirectory = ".";
            saveFileDialog1.Filter = "Windows Bitmap (*.bmp) | *.bmp"
                                   + "| JPEG (*.jpg) | *.jpg"
                                   + "| PNG (*.png) | *.png";
            saveFileDialog1.FilterIndex = 1;
            saveFileDialog1.RestoreDirectory = true;
            
            // Drucker
            printDocument1.PrinterSettings.Duplex = Duplex.Simplex;
       }

        private void ffnenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                img = Image.FromFile(openFileDialog1.FileName);
                this.Refresh();
                
                // Druckuntersttzung
                printDocument1.DocumentName = openFileDialog1.FileName;
            }
        }

        private void speichernToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (img == null)
                return;
                                
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                if (saveFileDialog1.FileName == openFileDialog1.FileName)
                {
                    MessageBox.Show("Zurckschreiben in die alte Datei wird nicht untersttzt");
                    return;
                }
               
                switch (saveFileDialog1.FilterIndex)
                {
                case 1 : img.Save(saveFileDialog1.FileName, ImageFormat.Bmp);    
                         break;
                case 2 : img.Save(saveFileDialog1.FileName, ImageFormat.Jpeg);    
                         break;
                case 3 : img.Save(saveFileDialog1.FileName, ImageFormat.Png);    
                         break;
                } 
                
                                
                // Druckuntersttzung
                printDocument1.DocumentName = openFileDialog1.FileName;        
            }            
        }

        private void beendenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void panel1_Paint(object sender, PaintEventArgs e)
        {
            if (img != null)
            {
                Rectangle rect = new Rectangle(panel1.ClientSize.Width/2 - img.Width/2, 
                                               panel1.ClientSize.Height/2 - img.Height/2,
                                               img.Width, img.Height);
                                               
                double ratioX = (double) panel1.ClientSize.Width/img.Width;
                double ratioY = (double) panel1.ClientSize.Height/img.Height;
                
                if (ratioX < 1 && ratioX < ratioY)
                {
                    rect.Height = (int) (img.Height*ratioX);
                    rect.Width = (int) (img.Width*ratioX);
                    rect.Y = panel1.ClientSize.Height/2 - rect.Height/2;
                    rect.X = panel1.ClientSize.Width/2 - rect.Width/2;
                }
                if (ratioY < 1 && ratioY < ratioX)
                {
                    rect.Height = (int) (img.Height*ratioY);
                    rect.Width = (int) (img.Width*ratioY);
                    rect.Y = panel1.ClientSize.Height/2 - rect.Height/2;
                    rect.X = panel1.ClientSize.Width/2 - rect.Width/2;
                }
               
                e.Graphics.DrawImage(img, rect);
            }
        }

        private void signierenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (img == null)
                return;
                
            if (img.Width > 100 && img.Height > 50)
            {
                Bitmap sign = Bildsignierer.Properties.Resources.signatur;
                Graphics g = Graphics.FromImage(img);
                
                Rectangle r = new Rectangle(img.Width - sign.Width - 10,
                                            img.Height - sign.Height - 10,
                                            sign.Width, sign.Height);
                
                if (sign.Width > img.Width/5)
                {
                    r.Width = img.Width/5;
                    r.X = img.Width - r.Width - 5;
                }
                if (sign.Height > img.Height/5)
                {
                    r.Width = img.Height/5;
                    r.Y = img.Height - r.Height - 5;
                }
                
                g.DrawImage(sign, r);
                panel1.Refresh();
            }
        }

        private void graustufenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (img != null && img is Bitmap)
            {
                ImageAttributes iA = new ImageAttributes();
                int w = img.Width;
                int h = img.Height;
                
                BufferedGraphicsContext bgc = BufferedGraphicsManager.Current;
                BufferedGraphics bg = bgc.Allocate(Graphics.FromImage(img), new Rectangle(0, 0, w, h));
            
                float[][] cmElements = { 
                   new float[] {0.30F,  0.30F,  0.30F,  0, 0},
                   new float[] {0.59F,  0.59F,  0.59F,  0, 0},
                   new float[] {0.11F,  0.11F,  0.11F,  0, 0},
                   new float[] {0, 0, 0, 1, 0},
                   new float[] {0, 0, 0, 0, 1}};
                ColorMatrix cm = new ColorMatrix(cmElements);

                iA.SetColorMatrix(cm, ColorMatrixFlag.Default, ColorAdjustType.Bitmap);

                bg.Graphics.DrawImage(img, 
                                      new Rectangle(0, 0, w, h),
                                      0, 0, w, h, GraphicsUnit.Pixel, iA);   
                   
                bg.Render();
                bg.Dispose();

                panel1.Refresh();
            }
        }

        private void spiegelnUmYAchseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (img != null && img is Bitmap)
            {
                img.RotateFlip(RotateFlipType.RotateNoneFlipX);
                panel1.Refresh();
            }
        }

        private void spiegelnUmXAchseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (img != null && img is Bitmap)
            {
                img.RotateFlip(RotateFlipType.RotateNoneFlipY);
                panel1.Refresh();
            }
        }

        private void druckenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (img != null)
            {
                // Schalte auf Querformat um, wenn das Bild breiter als hoch ist 
                // und die Breite grer ist als die Breite der bedruckbaren Flche 
                if (   img.Width > img.Height
                    && img.Width > printDocument1.DefaultPageSettings.PrintableArea.Width)
                {
                    printDocument1.DefaultPageSettings.Landscape = true;
                }
                else
                {
                    printDocument1.DefaultPageSettings.Landscape = false;
                }
               
                // Drucken
                printDocument1.Print();
            }
        }

        private void printDocument1_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {
            // Bild auf den bedruckbaren Bereich skalieren

            Rectangle rect = new Rectangle(e.MarginBounds.X, e.MarginBounds.Y, 
                                           img.Width, img.Height);
                                           
            double ratioX = (double) e.MarginBounds.Width/img.Width;
            double ratioY = (double) e.MarginBounds.Height/img.Height;
            
            if (ratioX < 1 && ratioX < ratioY)
            {
                rect.Height = (int) (img.Height*ratioX);
                rect.Width = (int) (img.Width*ratioX);
            }
            if (ratioY < 1 && ratioY < ratioX)
            {
                rect.Height = (int) (img.Height*ratioY);
                rect.Width = (int) (img.Width*ratioY);
            }
           
            e.Graphics.DrawImage(img, rect);
        }

    }
}