using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Drawing.Printing;

namespace Texteditor
{
    public partial class Form1 : Form
    {      
        private bool dirty;             
        private string filename;
        private Timer clipboardTimer;
        
        private OpenFileDialog openFileDialog1;
        private SaveFileDialog saveFileDialog1;
        public SearchDialog searchDlg;
        
        private string textToPrint;
        private int pageToPrint;

          
        public Form1()
        {
            InitializeComponent();
            
            filename = null;
            textBox1.Text = "";
            dirty = false;
            setFormTitle();
            
            // Deaktivierung der Datei-Menbefehle
            miEditCut.Enabled = false;
            miEditCopy.Enabled = false;
            miEditPaste.Enabled = false;

            // Erzeugung und Konfiguration der ffnen- und Speichern-Dialoge
            openFileDialog1 = new OpenFileDialog();
            openFileDialog1.InitialDirectory = ".";
            openFileDialog1.Filter = "CS-Dateien (*.cs)|*.cs|Textdateien (*.txt)|*.txt";
            openFileDialog1.FilterIndex = 1;
            openFileDialog1.Multiselect = false;
            openFileDialog1.RestoreDirectory = true;

            saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.InitialDirectory = ".";
            saveFileDialog1.Filter = "CS-Dateien (*.cs)|*.cs|Textdateien (*.txt)|*.txt";
            saveFileDialog1.FilterIndex = 1;
            saveFileDialog1.RestoreDirectory = true;
            
            // Such-Dialog
            searchDlg = null;            

            // Timer zur Aktivierung/Deaktivierung der Zwischenablagebefehle 
            // sowie zur Aktualisierung der Uhrzeit in der Statusleiste
            clipboardTimer = new Timer();
            clipboardTimer.Interval = 200;
            clipboardTimer.Tick += new EventHandler(this.OnTimer);
            clipboardTimer.Start();
        }
        
        
        /*
         * Ereignisbehandlung und Hilfsmethoden fr die Befehle im Men Datei 
         */
         
        private bool nothingToSave()
        {
            if (!dirty)
            {
                return true;
            }
            
            switch (MessageBox.Show("nderungen speichern?", "Texteditor",
                                    MessageBoxButtons.YesNoCancel))
            {
            case DialogResult.Yes:    return save(filename);
            case DialogResult.No:     return true;
            case DialogResult.Cancel:
            default:                  return false;
            }
        }
        
        private void setFormTitle()
        {
            string title;
            
            if (filename == null)
                title = "Unbenannt";
            else
            {
                // Nur den reinen Dateinamen ohne Pfad bernehmen
                title = filename.Substring(filename.LastIndexOf('\\')+1);
            }
                
            this.Text = "Texteditor - " + title;
        }

        private void miFileNew_Click(object sender, EventArgs e)
        {
            if (nothingToSave())
            {
                // Inhalt des Textfelds lschen
                textBox1.Text = "";
                
                // kein Dateiname
                filename = null;
                
                // Keine ungespeicherten nderungen
                dirty = false;
                
                // Fensteroberflche aktualisieren
                miFileSave.Enabled = false;
                miFileSaveAs.Enabled = false;
                btnSave.Enabled = false;
                setFormTitle();
                sbToolTipField.Text = "Neues Dokument wurde angelegt";
            }
        }
 
        private void miFileOpen_Click(object sender, EventArgs e)
        {
            if (!nothingToSave())
                return;
                       
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                // Dateiinhalt einlesen
                open(openFileDialog1.FileName);                
            }
        }
        
        private void open(string filename)
        {
            try
            {
                StreamReader sr = new StreamReader(filename, 
                                                   Encoding.GetEncoding("windows-1252"), true);
                textBox1.Text = sr.ReadToEnd();
                sr.Close();
                
                // Fensteroberflche und interne Felder aktualisieren
                this.filename = filename;
                dirty = false;
                miFileSave.Enabled = false;
                miFileSaveAs.Enabled = false;
                btnSave.Enabled = false;
                setFormTitle();
                sbToolTipField.Text = filename + " wurde geffnet.";
            }
            catch (IOException)
            {
                MessageBox.Show("Datei konnte nicht geffnet werden.", "Fehler");
            }
        }
        
        private void miFileSave_Click(object sender, EventArgs e)
        {
            save(filename);
        }
                
        private void miFileSaveAs_Click(object sender, EventArgs e)
        {
            save(null);
        }
        
        private bool save(String filename)
        {
            if (filename == null)
            {
                if (saveFileDialog1.ShowDialog() == DialogResult.OK)
                    filename = saveFileDialog1.FileName;                
                else
                    return false;
            }
            
            try
            {
                StreamWriter sw = new StreamWriter(filename, false, 
                                                   Encoding.GetEncoding("windows-1252"));

                sw.Write(textBox1.Text);
                sw.Close();
                
                // Fensteroberflche und interne Felder aktualisieren
                this.filename = filename;
                dirty = false;
                miFileSave.Enabled = false;
                miFileSaveAs.Enabled = false;
                btnSave.Enabled = false;
                setFormTitle();
                sbToolTipField.Text = filename + " wurde geffnet.";
                
                return true;
            }
            catch (IOException)
            {
                MessageBox.Show("Datei konnte nicht gespeichert werden.", "Fehler");
            }
            
            return false;
        }

        private void miFilePageLayout_Click(object sender, EventArgs e)
        {
            // Druckerauftrag konfigurieren 
            printDocument1.DocumentName = filename;
            printDocument1.PrinterSettings.PrinterName = null;      // Standardddrucker
            printDocument1.PrinterSettings.Duplex = Duplex.Simplex; // kein beidseitiger Druck
            printDocument1.DefaultPageSettings.Margins = new Margins((int)(2/0.0254), 
                                                                     (int)(1/0.0254), 
                                                                     (int)(1/0.0254), 
                                                                     (int)(2/0.0254));

            // Seite einrichten-Dialog mit PrintDocument verbinden
            pageSetupDialog1.Document = printDocument1;
            
            // Seite einrichten-Dialog konfigurieren
            pageSetupDialog1.EnableMetric = true;
            
            // Seite einrichten-Dialog anzeigen und ggf. Druck starten
            if (pageSetupDialog1.ShowDialog() == DialogResult.OK)
            {   
                textToPrint = textBox1.Text;
                pageToPrint = 1;
                printDocument1.Print();
            }
        }

        private void miFilePrintPreview_Click(object sender, EventArgs e)
        {
            // Druckerauftrag konfigurieren 
            textToPrint = textBox1.Text;
            printDocument1.DocumentName = filename;
            printDocument1.PrinterSettings.PrinterName = null;      // Standardddrucker
            printDocument1.PrinterSettings.Duplex = Duplex.Simplex; // kein beidseitiger Druck
            printDocument1.DefaultPageSettings.Margins = new Margins((int)(2/0.0254), 
                                                                     (int)(1/0.0254), 
                                                                     (int)(1/0.0254), 
                                                                     (int)(2/0.0254));
            
            // Seitenansicht-Dialog mit PrintDocument verbinden
            printPreviewDialog1.Document = printDocument1;
            
            // Drucken-Dialog konfigurieren
            printPreviewDialog1.UseAntiAlias = true;
            
            // Drucken-Dialog anzeigen und ggf. Druck starten
            pageToPrint = 1;
            printPreviewDialog1.ShowDialog();            
        }
        
        private void miFilePrint_Click(object sender, EventArgs e)
        {
            // Druckerauftrag konfigurieren 
            printDocument1.DocumentName = filename;
            printDocument1.PrinterSettings.PrinterName = null;      // Standardddrucker
            printDocument1.PrinterSettings.Duplex = Duplex.Simplex; // kein beidseitiger Druck
            printDocument1.DefaultPageSettings.Margins = new Margins((int)(2/0.0254), 
                                                                     (int)(1/0.0254), 
                                                                     (int)(1/0.0254), 
                                                                     (int)(2/0.0254));

            // Drucken-Dialog mit PrintDocument verbinden
            printDialog1.Document = printDocument1;
            
            // Drucken-Dialog konfigurieren
            printDialog1.AllowPrintToFile = false;
            printDialog1.AllowSelection = true;
            
            // Drucken-Dialog anzeigen und ggf. Druck starten
            if (printDialog1.ShowDialog() == DialogResult.OK)
            {
                if (printDocument1.PrinterSettings.PrintRange == PrintRange.AllPages)
                {
                    textToPrint = textBox1.Text;
                }
                else if (printDocument1.PrinterSettings.PrintRange == PrintRange.Selection)
                {
                    textToPrint = textBox1.SelectedText;
                }
                                
                pageToPrint = 1;
                printDocument1.Print();
            }
        }

        private void miFileQuit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            if (!dirty)
            {
                dirty = true;
                
                miFileSave.Enabled = true;
                miFileSaveAs.Enabled = true;
                btnSave.Enabled = true;
            }                        
        }


        /*
         * Ereignisbehandlung und Hilfsmethoden fr die Befehle im Men Bearbeiten 
         */
         
        private void miEditCut_Click(object sender, EventArgs e)
        {
            if (textBox1.SelectionLength > 0)
            {
                Clipboard.SetText(textBox1.SelectedText);
                textBox1.SelectedText = "";
            }
        }

        private void miEditCopy_Click(object sender, EventArgs e)
        {
            if (textBox1.SelectionLength > 0)
            {
                Clipboard.SetData(DataFormats.Text, textBox1.SelectedText);
                
                /* Demonstration zum Einfgen von Daten fr mehrere Formate
                String s = textBox1.SelectedText;
                DataObject data = new DataObject();
                data.SetData(DataFormats.Text, s);
                data.SetData(DataFormats.Html, s);
                
                
                Font font = new Font("Arial", 12);
                Size size = TextRenderer.MeasureText(s, font);
                Bitmap pic = new Bitmap(size.Width + 20, size.Height);
                Graphics g = Graphics.FromImage(pic);
                g.Clear(Color.White);
                TextRenderer.DrawText(g, s, font, new Point(0,0), Color.Black);
                data.SetData(DataFormats.Bitmap, pic);
                
                Clipboard.SetDataObject(data);
                */
            }
        }

        private void miEditPaste_Click(object sender, EventArgs e)
        {
            if(Clipboard.ContainsText() == true)
            {
                textBox1.SelectedText = Clipboard.GetText();
            }
        }

        private void OnTimer(object sender, EventArgs args)
        {
            if (textBox1.SelectionLength > 0)
            {
                miEditCut.Enabled = true;
                miEditCopy.Enabled = true;
                btnCut.Enabled = true;
                btnCopy.Enabled = true;
            }
            else
            {
                miEditCut.Enabled = false;
                miEditCopy.Enabled = false;
                btnCut.Enabled = false;
                btnCopy.Enabled = false;
            }
            
            if (Clipboard.ContainsText() == true)
            {
                miEditPaste.Enabled = true;
                btnPaste.Enabled = true;
            }
            else
            {
                miEditPaste.Enabled = false;
                btnPaste.Enabled = false;
            }

            sbClockField.Text = DateTime.Now.ToShortTimeString();           
        }


        private void miEditSearch_Click(object sender, EventArgs e)
        {
            if (searchDlg == null)
                searchDlg = new SearchDialog(this);
            else
                searchDlg.Focus();
            
            searchDlg.Show();
        }



        /*
         * Hilfe zu Menbefehlen in Statusleiste einblenden 
         */
         
        private void menuItem_MouseHover(object sender, EventArgs e)
        {
            if (sender is ToolStripMenuItem)
            {
                sbToolTipField.Text = ((ToolStripMenuItem)sender).Text;
            }
            if (sender is ToolStripButton)
            {
                sbToolTipField.Text = ((ToolStripButton)sender).ToolTipText;
            }

        }

        private void menuItem_MouseLeave(object sender, EventArgs e)
        {
            sbToolTipField.Text = "";
        }



        /*
         * Drag & Drop-Untersttzung 
         */
         
        private void textBox1_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.Copy;
            else
                e.Effect = DragDropEffects.None;
        }

        private void textBox1_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                string[] filenames = (string[])e.Data.GetData(DataFormats.FileDrop);
                open(filenames[0]);
           }
        }



        /*
         * Drucken 
         */
         
        private void printDocument1_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {
            int charsFitted = 0;
            int linesFilled = 0;
            
            e.Graphics.MeasureString(textToPrint, this.Font, e.MarginBounds.Size, 
                                     StringFormat.GenericTypographic, 
                                     out charsFitted, out linesFilled);
                        
            e.Graphics.DrawString(textToPrint, this.Font, Brushes.Black, e.MarginBounds,
                                  StringFormat.GenericTypographic);
                                     
            String strPageNumber = "- " + pageToPrint + " -";
            SizeF size = e.Graphics.MeasureString(strPageNumber, this.Font);            
            e.Graphics.DrawString(strPageNumber, this.Font, Brushes.Black, 
                                  e.PageBounds.Width/2 - size.Width/2,
                                  e.MarginBounds.Bottom + 10);            
                  
            textToPrint = textToPrint.Substring(charsFitted);
            
            if (textToPrint.Length > 0)
            {
                e.HasMorePages = true;
                pageToPrint++;
            }
            else
            {
                pageToPrint = 1;
                textToPrint = textBox1.Text;
            }
                        
        }

    }
}
