using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;

namespace Bildbetrachter
{
    public partial class Form1 : Form
    {
        Image img;
        OpenFileDialog openFileDialog1;
        SaveFileDialog saveFileDialog1;
        
        public Form1()
        {
            InitializeComponent();
            ResizeRedraw = true;
            img = null;
            
            hScrollBar1.Parent = pictureBox1;
            vScrollBar1.Parent = pictureBox1;
            hScrollBar1.Visible = false;
            vScrollBar1.Visible = false;
 
            openFileDialog1 = new OpenFileDialog();
            openFileDialog1.InitialDirectory = ".";
            openFileDialog1.Filter = "Windows Bitmap (*.bmp) | *.bmp"
                                   + "| JPEG (*.jpg) | *.jpg"
                                   + "| PNG (*.png) | *.png"
                                   + "| Windows Metafile (*.wmf) | *.wmf"
                                   + "| Grafikdatei (*.bmp;*.jpg;*.png;*.wmf) | *.bmp;*.jpg;*.png;*.wmf";
            openFileDialog1.FilterIndex = 5;
            openFileDialog1.Multiselect = false;
            openFileDialog1.RestoreDirectory = true;
            
            saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.InitialDirectory = ".";
            saveFileDialog1.Filter = "Windows Bitmap (*.bmp) | *.bmp"
                                   + "| JPEG (*.jpg) | *.jpg"
                                   + "| PNG (*.png) | *.png";
            saveFileDialog1.FilterIndex = 1;
            saveFileDialog1.RestoreDirectory = true;
        }

        private void ffnenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                // Bild laden
                img = Image.FromFile(openFileDialog1.FileName);
                
                // Rendering des Bildes anstoen
                pictureBox1.Image = img;
                    
                // Bildlaufleisten einstellen
                setScrollBars();                    
            }
        }

        private void speichernToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (img == null)
                return;
                
            // voreingestellten Dateityp an Bildformat anpassen
            if (img.RawFormat.Equals(ImageFormat.Bmp))
                saveFileDialog1.FilterIndex = 1;    
            else if (img.RawFormat.Equals(ImageFormat.Jpeg))
                saveFileDialog1.FilterIndex = 2;
            else 
                saveFileDialog1.FilterIndex = 3;    
                
                
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                if (saveFileDialog1.FileName == openFileDialog1.FileName)
                {
                    MessageBox.Show("Zurckschreiben in die alte Datei wird nicht untersttzt");
                    return;
                }
               
                switch (saveFileDialog1.FilterIndex)
                {
                case 1 : img.Save(saveFileDialog1.FileName, ImageFormat.Bmp);    
                         break;
                case 2 : img.Save(saveFileDialog1.FileName, ImageFormat.Jpeg);    
                         break;
                case 3 : img.Save(saveFileDialog1.FileName, ImageFormat.Png);    
                         break;
                }         
            }            
        }

        private void beendenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.Clear(SystemColors.ControlLight);
            
            if (pictureBox1.Image != null)
            {
                // Korrektur fr mglicherweise eingeblendete Bildlaufleisten
                int corrW = 0;
                int corrH = 0;
                
                if (vScrollBar1.Visible)
                    corrW = vScrollBar1.Width;
                if (hScrollBar1.Visible)
                    corrH = hScrollBar1.Height;
                    
                e.Graphics.DrawImage(pictureBox1.Image, 
                                     new Rectangle(0, 0, 
                                                   pictureBox1.Right - corrW, 
                                                   pictureBox1.Bottom - corrH), 
                                     new Rectangle(hScrollBar1.Value, vScrollBar1.Value, 
                                                   pictureBox1.Right - corrW,
                                                   pictureBox1.Bottom - corrH),
                                     GraphicsUnit.Pixel);
            }
        }


        
        private void setScrollBars()
        {
            // Bildlaufleisten einblenden, wenn die Bildbreite (-hhe) grer als die PictureBox ist
            hScrollBar1.Visible = pictureBox1.Image.Width  > pictureBox1.Width;
            vScrollBar1.Visible = pictureBox1.Image.Height > pictureBox1.Height;
            
            // nochmals prfen und sichtbare Bildlaufleisten mit bercksichtigen
            if (vScrollBar1.Visible)
                hScrollBar1.Visible = pictureBox1.Image.Width  > (pictureBox1.Width - vScrollBar1.Width);
            if (hScrollBar1.Visible)
                vScrollBar1.Visible = pictureBox1.Image.Height > (pictureBox1.Height - hScrollBar1.Height);
 
            // horizontale Bildlaufleiste konfigurieren
            hScrollBar1.Value = 0;
            hScrollBar1.Minimum = 0;
            hScrollBar1.Maximum = pictureBox1.Image.Width - 1;
            hScrollBar1.SmallChange = 1;
            // LargeChange auf Anzeigebreite setzen
            if (hScrollBar1.Visible)
            {
                if (vScrollBar1.Visible)
                    hScrollBar1.LargeChange = pictureBox1.Width - vScrollBar1.Width;
                else
                    hScrollBar1.LargeChange = pictureBox1.Width;
            }            
            
            // vertikale Bildlaufleiste konfigurieren
            vScrollBar1.Value = 0;
            vScrollBar1.Minimum = 0;
            vScrollBar1.Maximum = pictureBox1.Image.Height - 1;
            vScrollBar1.SmallChange = 1;         
            // LargeChange auf Anzeigehhe setzen
            if (vScrollBar1.Visible)
            {
                if (hScrollBar1.Visible)
                    vScrollBar1.LargeChange = pictureBox1.Height - hScrollBar1.Height;
                else
                    vScrollBar1.LargeChange = pictureBox1.Height;
            }
     
        }

        private void vScrollBar1_Scroll(object sender, ScrollEventArgs e)
        {
            pictureBox1.Refresh();
        }

        private void hScrollBar1_Scroll(object sender, ScrollEventArgs e)
        {
            pictureBox1.Refresh();
        }

        private void pictureBox1_Resize(object sender, EventArgs e)
        {
            // Nicht reagieren, wenn kein Bild geladen oder das Fenster minimiert wird
            if (pictureBox1.Image != null && WindowState != FormWindowState.Minimized)
            {
                setScrollBars();
                pictureBox1.Refresh();
            }
        }
    }
}